#!/usr/bin/env python
# -*- coding: utf-8 -*-

# subtract one Digital Elevation Model dataset from another to produce 
# a third. e.g. subtract DTM from DSM to examine building heights

import sys
import os.path
import argparse

__version__='0.1'
__progdesc__='subtract one DEM file from another'

def get_args(parser):
	'''parse the command line'''
	parser.add_argument('dem1', help='first DEM file')
	parser.add_argument('dem2', help='second DEM file')
	parser.add_argument('outfile', help='output DEM file')
	parser.add_argument('sqlout', help='output SQL')
	info = parser.add_mutually_exclusive_group()
	info.add_argument('-q','--quiet', action='store_true', default=False, dest='quiet', help='process quietly')
	info.add_argument('-v','--verbose', action='store_true', default=False, dest='verbose', help='verbose, overrides quiet (-q)')
	parser.add_argument('--version', action='version', version='%(prog)s version {0}'.format(__version__))
	
	results = parser.parse_args()
	return results

def main():
	# sort out the command line stuff
	parser = argparse.ArgumentParser(description=__progdesc__)
	arga=get_args(parser)
	if arga.verbose:
		# display some info
		print('{0} Version {1}'.format(sys.argv[0], __version__))
		print(__progdesc__)
	
	# do the input files exist? 
	if os.path.isfile(arga.dem1):
		pass
	else:
		print('File {0} not found'.format(arga.dem1))
		sys.exit(8)
	if os.path.isfile(arga.dem1):
		pass
	else:
		print('File {0} not found'.format(arga.dem2))
		sys.exit(8)
	
	if arga.quiet:
		pass
	else:
		print('Subtracting {0} from {1}'.format(arga.dem2, arga.dem1))
	
	# process the files
	with open(arga.dem1, 'rb') as fdem1, open(arga.dem2,"rb") as fdem2, open(arga.outfile, "w") as fout, open(arga.sqlout, "w") as fsql:
		# read the dem1 & dem2 headers - they need to match
		l = fdem1.readline().split()[1]
		d1ncols = int(l)
		l = fdem1.readline().split()[1]
		d1nrows = int(l)
		l = fdem1.readline().split()[1]
		d1xll = float(l)
		l = fdem1.readline().split()[1]
		d1yll = float(l)
		l = fdem1.readline().split()[1]
		d1cellsize = float(l)
		l = fdem1.readline().split()[1]
		d1nodata = int(l)
		if arga.verbose:
			print ('dem1 cols {0} rows {1} x {2} y {3} size {4} nodata {5}'.format(d1ncols,d1nrows,d1xll,d1yll,d1cellsize,d1nodata));
		
		l = fdem2.readline().split()[1]
		d2ncols = int(l)
		l = fdem2.readline().split()[1]
		d2nrows = int(l)
		l = fdem2.readline().split()[1]
		d2xll = float(l)
		l = fdem2.readline().split()[1]
		d2yll = float(l)
		l = fdem2.readline().split()[1]
		d2cellsize = float(l)
		l = fdem2.readline().split()[1]
		d2nodata = int(l)
		if arga.verbose:
			print ('dem2 cols {0} rows {1} x {2} y {3} size {4} nodata {5}'.format(d2ncols,d2nrows,d2xll,d2yll,d2cellsize,d2nodata));
		
		# compare dem1 & dem2. If not the same, end with error
		if d2ncols != d1ncols:
			print('ncols do not match')
			sys.exit(2)
		if d2nrows != d1nrows:
			print('nrows do not match')
			sys.exit(2)
		if d2xll != d1xll:
			print('xllcorners do not match')
			sys.exit(2)
		if d2yll != d1yll:
			print('yllcorners do not match')
			sys.exit(2)
		if d2cellsize != d1cellsize:
			print('cellsizes do not match')
			sys.exit(2)
		if d2nodata != d1nodata:
			print('nodatas do not match')
			sys.exit(2)
		
		# write the header to the output file. Use dem1 as files match
		fout.write('ncols        {0}\n'.format(d1ncols))
		fout.write('nrows        {0}\n'.format(d1nrows))
		fout.write('xllcorner    {0}\n'.format(d1xll))
		fout.write('yllcorner    {0}\n'.format(d1yll))
		fout.write('cellsize     {0}\n'.format(d1cellsize))
		fout.write('NODATA_value {0}\n'.format(d1nodata))
		
		# process the lines of data. Write new dem file & SQL inserts
		# batch the inserts for each row for quicker loading
		y = d1yll + (d1cellsize * d1nrows)
		for r in range(d1nrows):
			l1 = fdem1.readline().split()
			l2 = fdem2.readline().split()
			fsql.write("INSERT INTO eadata(height,locn) VALUES")
			sep="\n"
			for k,h in enumerate(l1):
				if h==d1nodata:
					dif = d1nodata
				else:
					dif = float(h) - float(l2[k])
				if arga.verbose:
					print k,h,l2[k],dif
				fout.write(' {0}'.format(dif))
				x = d1xll + (k * d1cellsize)
				fsql.write("{0}({1}, ST_GeomFromText('POINT({2} {3})', 27700))".format(sep,dif,x,y))
				sep=",\n"
			fout.write('\n')
			fsql.write(";\n")
			y -= d1cellsize
		
	return 0

if __name__ == '__main__':
	main()

